@@ -21,14 +21,18 @@ function createWasmAudioWorkletProcessor(audioParams) {
 
       // Capture the Wasm function callback to invoke.
       let opts = args.processorOptions;
-      this.callbackFunction = wasmTable.get(opts['cb']);
-      this.userData = opts['ud'];
+#if ASSERTIONS
+      assert(opts.callback)
+      assert(opts.samplesPerChannel)
+#endif
+      this.callback = {{{ makeDynCall('iipipipp', 'opts.callback') }}};
+      this.userData = opts.userData;
       // Then the samples per channel to process, fixed for the lifetime of the
       // context that created this processor. Note for when moving to Web Audio
       // 1.1: the typed array passed to process() should be the same size as this
       // 'render quantum size', and this exercise of passing in the value
-      // shouldn't be required (to be verified).
-      this.samplesPerChannel = opts['sc'];
+      // shouldn't be required (to be verified)
+      this.samplesPerChannel = opts.samplesPerChannel;
     }
 
     static get parameterDescriptors() {
@@ -43,78 +47,86 @@ function createWasmAudioWorkletProcessor(audioParams) {
       // then perform the wasm audio worklet call,
       // and finally marshal audio output data back.
 
-      let numInputs = inputList.length,
-        numOutputs = outputList.length,
-        numParams = 0, i, j, k, dataPtr,
-        bytesPerChannel = this.samplesPerChannel * 4,
-        stackMemoryNeeded = (numInputs + numOutputs) * {{{ C_STRUCTS.AudioSampleFrame.__size__ }}},
-        oldStackPtr = stackSave(),
-        inputsPtr, outputsPtr, outputDataPtr, paramsPtr,
-        didProduceAudio, paramArray;
+      var numInputs = inputList.length;
+      var numOutputs = outputList.length;
+
+      var entry; // reused list entry or index
+      var subentry; // reused channel or other array in each list entry or index
 
       // Calculate how much stack space is needed.
-      for (i of inputList) stackMemoryNeeded += i.length * bytesPerChannel;
-      for (i of outputList) stackMemoryNeeded += i.length * bytesPerChannel;
-      for (i in parameters) stackMemoryNeeded += parameters[i].byteLength + {{{ C_STRUCTS.AudioParamFrame.__size__ }}}, ++numParams;
+      var bytesPerChannel = this.samplesPerChannel * {{{ getNativeTypeSize('float') }}};
+      var stackMemoryNeeded = (numInputs + numOutputs) * {{{ C_STRUCTS.AudioSampleFrame.__size__ }}};
+      var numParams = 0;
+      for (entry of inputList) stackMemoryNeeded += entry.length * bytesPerChannel;
+      for (entry of outputList) stackMemoryNeeded += entry.length * bytesPerChannel;
+      for (entry in parameters) {
+        stackMemoryNeeded += parameters[entry].byteLength + {{{ C_STRUCTS.AudioParamFrame.__size__ }}};
+        ++numParams;
+      }
 
       // Allocate the necessary stack space.
-      inputsPtr = stackAlloc(stackMemoryNeeded);
+      var oldStackPtr = stackSave();
+      var inputsPtr = stackAlloc(stackMemoryNeeded);
 
-      // Copy input audio descriptor structs and data to Wasm
-      k = inputsPtr >> 2;
-      dataPtr = inputsPtr + numInputs * {{{ C_STRUCTS.AudioSampleFrame.__size__ }}};
-      for (i of inputList) {
+      // Copy input audio descriptor structs and data to Wasm ('structPtr' is
+      // reused as the working start to each struct record, 'dataPtr' start of
+      // the data section, usually after all structs).
+      var structPtr = inputsPtr;
+      var dataPtr = inputsPtr + numInputs * {{{ C_STRUCTS.AudioSampleFrame.__size__ }}};
+      for (entry of inputList) {
         // Write the AudioSampleFrame struct instance
-        HEAPU32[k + {{{ C_STRUCTS.AudioSampleFrame.numberOfChannels / 4 }}}] = i.length;
-        HEAPU32[k + {{{ C_STRUCTS.AudioSampleFrame.samplesPerChannel / 4 }}}] = this.samplesPerChannel;
-        HEAPU32[k + {{{ C_STRUCTS.AudioSampleFrame.data / 4 }}}] = dataPtr;
-        k += {{{ C_STRUCTS.AudioSampleFrame.__size__ / 4 }}};
+        {{{ makeSetValue('structPtr', C_STRUCTS.AudioSampleFrame.numberOfChannels, 'entry.length', 'u32') }}};
+        {{{ makeSetValue('structPtr', C_STRUCTS.AudioSampleFrame.samplesPerChannel, 'this.samplesPerChannel', 'u32') }}};
+        {{{ makeSetValue('structPtr', C_STRUCTS.AudioSampleFrame.data, 'dataPtr', '*') }}};
+        structPtr += {{{ C_STRUCTS.AudioSampleFrame.__size__ }}};
         // Marshal the input audio sample data for each audio channel of this input
-        for (j of i) {
-          HEAPF32.set(j, dataPtr>>2);
+        for (subentry of entry) {
+          HEAPF32.set(subentry, {{{ getHeapOffset('dataPtr', 'float') }}});
           dataPtr += bytesPerChannel;
         }
       }
 
       // Copy output audio descriptor structs to Wasm
-      outputsPtr = dataPtr;
-      k = outputsPtr >> 2;
-      outputDataPtr = (dataPtr += numOutputs * {{{ C_STRUCTS.AudioSampleFrame.__size__ }}}) >> 2;
-      for (i of outputList) {
+      var outputsPtr = dataPtr;
+      structPtr = outputsPtr;
+      var outputDataPtr = (dataPtr += numOutputs * {{{ C_STRUCTS.AudioSampleFrame.__size__ }}});
+      for (entry of outputList) {
         // Write the AudioSampleFrame struct instance
-        HEAPU32[k + {{{ C_STRUCTS.AudioSampleFrame.numberOfChannels / 4 }}}] = i.length;
-        HEAPU32[k + {{{ C_STRUCTS.AudioSampleFrame.samplesPerChannel / 4 }}}] = this.samplesPerChannel;
-        HEAPU32[k + {{{ C_STRUCTS.AudioSampleFrame.data / 4 }}}] = dataPtr;
-        k += {{{ C_STRUCTS.AudioSampleFrame.__size__ / 4 }}};
+        {{{ makeSetValue('structPtr', C_STRUCTS.AudioSampleFrame.numberOfChannels, 'entry.length', 'u32') }}};
+        {{{ makeSetValue('structPtr', C_STRUCTS.AudioSampleFrame.samplesPerChannel, 'this.samplesPerChannel', 'u32') }}};
+        {{{ makeSetValue('structPtr', C_STRUCTS.AudioSampleFrame.data, 'dataPtr', '*') }}};
+        structPtr += {{{ C_STRUCTS.AudioSampleFrame.__size__ }}};
         // Reserve space for the output data
-        dataPtr += bytesPerChannel * i.length;
+        dataPtr += bytesPerChannel * entry.length;
       }
 
       // Copy parameters descriptor structs and data to Wasm
-      paramsPtr = dataPtr;
-      k = paramsPtr >> 2;
+      var paramsPtr = dataPtr;
+      structPtr = paramsPtr;
       dataPtr += numParams * {{{ C_STRUCTS.AudioParamFrame.__size__ }}};
-
-      for (i = 0; paramArray = parameters[i++];) {
+      for (entry = 0; subentry = parameters[entry++];) {
         // Write the AudioParamFrame struct instance
-        HEAPU32[k + {{{ C_STRUCTS.AudioParamFrame.length / 4 }}}] = paramArray.length;
-        HEAPU32[k + {{{ C_STRUCTS.AudioParamFrame.data / 4 }}}] = dataPtr;
-        k += {{{ C_STRUCTS.AudioParamFrame.__size__ / 4 }}};
+        {{{ makeSetValue('structPtr', C_STRUCTS.AudioParamFrame.length, 'subentry.length', 'u32') }}};
+        {{{ makeSetValue('structPtr', C_STRUCTS.AudioParamFrame.data, 'dataPtr', '*') }}};
+        structPtr += {{{ C_STRUCTS.AudioParamFrame.__size__ }}};
         // Marshal the audio parameters array
-        HEAPF32.set(paramArray, dataPtr>>2);
-        dataPtr += paramArray.length*4;
+        HEAPF32.set(subentry, {{{ getHeapOffset('dataPtr', 'float') }}});
+        dataPtr += subentry.length * {{{ getNativeTypeSize('float') }}};
       }
 
       // Call out to Wasm callback to perform audio processing
-      if (didProduceAudio = this.callbackFunction(numInputs, inputsPtr, numOutputs, outputsPtr, numParams, paramsPtr, this.userData)) {
+      var didProduceAudio = this.callback(numInputs, inputsPtr, numOutputs, outputsPtr, numParams, paramsPtr, this.userData);
+      if (didProduceAudio) {
         // Read back the produced audio data to all outputs and their channels.
         // (A garbage-free function TypedArray.copy(dstTypedArray, dstOffset,
         // srcTypedArray, srcOffset, count) would sure be handy..  but web does
         // not have one, so manually copy all bytes in)
-        for (i of outputList) {
-          for (j of i) {
-            for (k = 0; k < this.samplesPerChannel; ++k) {
-              j[k] = HEAPF32[outputDataPtr++];
+        outputDataPtr = {{{ getHeapOffset('outputDataPtr', 'float') }}};
+        for (entry of outputList) {
+          for (subentry of entry) {
+            // repurposing structPtr for now
+            for (structPtr = 0; structPtr < this.samplesPerChannel; ++structPtr) {
+              subentry[structPtr] = HEAPF32[outputDataPtr++];
             }
           }
         }
@@ -138,7 +150,7 @@ var messagePort;
 class BootstrapMessages extends AudioWorkletProcessor {
   constructor(arg) {
     super();
-    startWasmWorker(arg['processorOptions'])
+    startWasmWorker(arg.processorOptions)
 #if WEBAUDIO_DEBUG
     console.log('AudioWorklet global scope looks like this:');
     console.dir(globalThis);
@@ -154,28 +166,23 @@ class BootstrapMessages extends AudioWorkletProcessor {
         // '_wpn' is short for 'Worklet Processor Node', using an identifier
         // that will never conflict with user messages
         // Register a real AudioWorkletProcessor that will actually do audio processing.
-        // 'ap' being the audio params
-        registerProcessor(d['_wpn'], createWasmAudioWorkletProcessor(d['ap']));
+        registerProcessor(d['_wpn'], createWasmAudioWorkletProcessor(d.audioParams));
 #if WEBAUDIO_DEBUG
-        console.log(`Registered a new WasmAudioWorkletProcessor "${d['_wpn']}" with AudioParams: ${d['ap']}`);
+        console.log(`Registered a new WasmAudioWorkletProcessor "${d['_wpn']}" with AudioParams: ${d.audioParams}`);
 #endif
         // Post a Wasm Call message back telling that we have now registered the
         // AudioWorkletProcessor, and should trigger the user onSuccess callback
         // of the emscripten_create_wasm_audio_worklet_processor_async() call.
         //
         // '_wsc' is short for 'wasm call', using an identifier that will never
-        // conflict with user messages
-        // 'cb' the callback function
-        // 'ch' the context handle
-        // 'ud' the passed user data
-        messagePort.postMessage({'_wsc': d['cb'], 'x': [d['ch'], 1/*EM_TRUE*/, d['ud']] });
+        // conflict with user messages.
+        //
+        // Note: we convert the pointer arg manually here since the call site
+        // ($_EmAudioDispatchProcessorCallback) is used with various signatures
+        // and we do not know the types in advance.
+        messagePort.postMessage({'_wsc': d.callback, args: [d.contextHandle, 1/*EM_TRUE*/, {{{ to64('d.userData') }}}] });
       } else if (d['_wsc']) {
-#if MEMORY64
-        var ptr = BigInt(d['_wsc']);
-#else
-        var ptr = d['_wsc'];
-#endif
-        wasmTable.get(ptr)(...d['x']);
+        getWasmTableEntry(d['_wsc'])(...d.args);
       };
     }
   }
