@@ -6,7 +6,6 @@
 
 var LibraryFS = {
   $FS__deps: ['$randomFill', '$PATH', '$PATH_FS', '$TTY', '$MEMFS',
-    '$FS_createPreloadedFile',
     '$FS_modeStringToFlags',
     '$FS_getMode',
     '$intArrayFromString',
@@ -27,6 +26,9 @@ var LibraryFS = {
 #endif
 #if ASSERTIONS
     '$strError', '$ERRNO_CODES',
+#endif
+#if !MINIMAL_RUNTIME
+    '$FS_createPreloadedFile',
 #endif
   ],
   $FS__postset: () => {
@@ -35,7 +37,10 @@ var LibraryFS = {
     addAtPostCtor('FS.ignorePermissions = false;');
     addAtExit('FS.quit();');
     return `
+#if !MINIMAL_RUNTIME
 FS.createPreloadedFile = FS_createPreloadedFile;
+FS.preloadFile = FS_preloadFile;
+#endif
 FS.staticInit();`;
   },
   $FS: {
@@ -197,6 +202,9 @@ FS.staticInit();`;
             current_path = PATH.dirname(current_path);
             if (FS.isRoot(current)) {
               path = current_path + '/' + parts.slice(i + 1).join('/');
+              // We're making progress here, don't let many consecutive ..'s
+              // lead to ELOOP
+              nlinks--;
               continue linkloop;
             } else {
               current = current.parent;
