@@ -409,6 +409,7 @@ def parse_args(newargs):  # noqa: C901, PLR0912, PLR0915
           settings.EMIT_NAME_SECTION = 1
         # In all cases set the emscripten debug level to 3 so that we do not
         # strip during link (during compile, this does not make a difference).
+        # TODO: possibly decouple some of these flags from the final debug level (#20462)
         settings.DEBUG_LEVEL = 3
     elif check_flag('-profiling') or check_flag('--profiling'):
       settings.DEBUG_LEVEL = max(settings.DEBUG_LEVEL, 2)
@@ -512,6 +513,12 @@ def parse_args(newargs):  # noqa: C901, PLR0912, PLR0915
       options.cpu_profiler = True
     elif check_flag('--threadprofiler'):
       settings_changes.append('PTHREADS_PROFILING=1')
+    elif arg in ('-fcolor-diagnostics', '-fdiagnostics-color', '-fdiagnostics-color=always'):
+      diagnostics.color_enabled = True
+    elif arg in ('-fno-color-diagnostics', '-fdiagnostics-color=never'):
+      diagnostics.color_enabled = False
+    elif arg == '-fansi-escape-codes':
+      diagnostics.force_ansi = True
     elif arg == '-fno-exceptions':
       settings.DISABLE_EXCEPTION_CATCHING = 1
       settings.DISABLE_EXCEPTION_THROWING = 1
